@@ -3,53 +3,23 @@
 #include "pmico.h"
 #include "exttypes.h"
 
-// A table connecting CORBA::Object_ptr's to the surrogate or real
+// A table connecting CORBA::Object_ptr's to the surrogate 
 // Perl object. We store the objects here as IV's, not as SV's,
 // since we don't hold a reference on the object, and need to
 // remove them from here when reference count has dropped to zero
 static HV *pin_table = 0;
 
-static const U32 instvars_magic = 0x18981972;
-
-// Magic (adopted from DBI) to attach InstVars invisibly to perlobj
-PMicoInstVars *
-pmico_instvars_add (SV *perlobj) 
-{
-    SV *iv_sv = newSV (sizeof(PMicoInstVars));
-    PMicoInstVars *iv = (PMicoInstVars *)SvPVX(iv_sv);
-
-    SV *rv = newRV(iv_sv);	// just needed for sv_bless
-    sv_bless (rv, gv_stashpv("CORBA::MICO::InstVars", TRUE));
-    sv_free (rv);
-
-    iv->magic = instvars_magic;
-    iv->obj = NULL;
-    iv->trueobj = NULL;
-    iv->repoid = NULL;
-
-    if (SvROK(perlobj))
-	perlobj = SvRV(perlobj);
-    
-    sv_magic (perlobj, iv_sv, '~' , Nullch, 0);
-    SvREFCNT_dec (iv_sv);	// sv_magic() incremented it
-    // It looks from sv.c like this is now unecessary, but DBI does it
-    // and it shouldn't do any harm
-    SvRMAGICAL_on (perlobj);
-
-    return iv;
-}
-
 // Find or create a Perl object for this CORBA object.
 // Takes over ownership of obj
 SV *
-pmico_find_or_create (CORBA::Object *obj)
+pmico_objref_to_sv (CORBA::Object *obj)
 {
     if (CORBA::is_nil (obj))
 	// FIXME: memory leaks?
 	return newSVsv(&sv_undef);
     
     char buf[24];
-    sprintf(buf, "%d", (IV)obj);
+    sprintf(buf, "%ld", (IV)obj);
 
     if (!pin_table)
 	pin_table = newHV();
@@ -61,8 +31,7 @@ pmico_find_or_create (CORBA::Object *obj)
 	}
     }
 
-    // We make surrogate objects hash refs - it might come in handy...
-    SV *result = newRV_noinc((SV *)newHV());
+    SV *result = newRV_noinc(newSViv((IV)obj));
 
     PMicoIfaceInfo *info = pmico_find_interface_description (obj->_repoid());
     if (info)
@@ -70,163 +39,23 @@ pmico_find_or_create (CORBA::Object *obj)
     else
 	sv_bless (result, gv_stashpv("CORBA::Object", TRUE));
 
-    PMicoInstVars *iv = pmico_instvars_add (result);
-
-    iv->obj = obj;
-    iv->trueobj = NULL;
-
     hv_store (pin_table, buf, strlen(buf), newSViv((IV)SvRV(result)), 0);
 
     return result;
 }
 
-PMicoInstVars *
-pmico_instvars_get (SV *perlobj) 
-{
-    PMicoInstVars *iv = NULL;
-    
-    if (SvROK(perlobj))
-	perlobj = SvRV(perlobj);
-
-    if (SvMAGICAL (perlobj)) {
-        MAGIC *mg = mg_find (perlobj, '~');
-    
-        if (mg)
-	    iv = (PMicoInstVars *)SvPVX(mg->mg_obj);
-    }
-
-    if (iv && (iv->magic == instvars_magic))
-	return iv;
-    else
-	return NULL;
-}
-
-const char *
-pmico_get_repoid (SV *perlobj, PMicoInstVars *iv)
-{
-    if (iv->repoid == NULL) {
-	dSP;
-	PUSHMARK(sp);
-	XPUSHs(perlobj);
-	PUTBACK;
-	
-	int count = perl_call_method("_pmico_repoid", G_SCALAR);
-	SPAGAIN;
-	
-	if (count != 1)			/* sanity check */
-	    croak("object->_pmico_repoid didn't return 1 argument");
-	
-	iv->repoid = new string (POPp);
-	
-	PUTBACK;
-    }
-
-    return iv->repoid->c_str();
-}
-
-PMicoInstVars *
-pmico_init_obj (SV *perlobj, 
-		CORBA::Object *o, 
-		CORBA::BOA::ReferenceData *refdata,
-		CORBA::InterfaceDef *iface, 
-		CORBA::ImplementationDef *impl,
-		const char *repoid) 
-{
-    PMicoInstVars *iv = pmico_instvars_get (perlobj);
-    CORBA::ImplementationDef_var implementation = CORBA::ImplementationDef::_duplicate(impl);
-
-    if (!iv) {
-	iv = pmico_instvars_add (perlobj);
-    }
-
-    if (!iv->obj) {
-	if (!repoid)
-	    repoid = pmico_get_repoid (perlobj, iv);
-	
-	PMicoIfaceInfo *info = pmico_find_interface_description (repoid);
-	if (!info)
-	    info = pmico_load_interface (NULL, NULL, repoid);
-
-	iv->trueobj = new PMicoTrueObject(perlobj, repoid);
-	iv->obj = iv->trueobj;
-
-	CORBA::BOA::ReferenceData *rd;
-	if (refdata)
-	    rd = refdata;
-	else
-	    rd = new CORBA::BOA::ReferenceData;
-
-	if (!implementation)
-	    implementation = iv->trueobj->_find_impl (iv->repoid->c_str(), info->desc->name);
-
-	assert ( !CORBA::is_nil (implementation) );
-
-	if (!iface)
-	    iface = info->iface;
-	
-	if (o)
-	    iv->trueobj->_restore_ref (o, *rd, iface, implementation);
-	else
-	    iv->trueobj->_create_ref (*rd, iface, implementation, repoid);
-
-	if (rd != refdata)
-	    delete rd;
-    }
-
-    if (!pin_table)
-	pin_table = newHV();
-
-    char buf[24];
-    sprintf(buf, "%d", (IV)iv->obj);
-
-    hv_store (pin_table, buf, strlen(buf), newSViv((IV)SvRV(perlobj)), 0);
-
-    return iv;
-}
-
 CORBA::Object_ptr
-pmico_sv_to_obj (SV *perlobj)
+pmico_sv_to_objref (SV *perlobj)
 {
     if (!SvOK(perlobj))
 	return CORBA::Object::_nil();
 
-    PMicoInstVars *iv = pmico_instvars_get (perlobj);
-    
-    if (!iv && !sv_derived_from (perlobj, "CORBA::Object"))
+    if (!sv_derived_from (perlobj, "CORBA::Object"))
 	croak ("Argument is not a CORBA::Object");
 
-    if (!iv || !iv->obj)
-	iv = pmico_init_obj (perlobj, NULL, NULL, NULL, NULL, NULL);
-    
-    return iv->obj;
+    return (CORBA::Object_ptr)SvIV((SV*)SvRV(perlobj));
 }
 
-void
-pmico_instvars_destroy (PMicoInstVars *instvars)
-{
-    char buf[24];
-    assert (instvars->magic == instvars_magic);
-
-    sprintf(buf, "%d", (IV)instvars->obj);
-
-    if (pin_table)
-	hv_delete(pin_table, buf, strlen(buf), G_DISCARD);
-
-    if (instvars->obj) {
-	if (instvars->trueobj)
-	    instvars->trueobj->servant_destroyed();
-
-	CORBA::release(instvars->obj);
-    }
-
-    if (instvars->repoid)
-	delete instvars->repoid;
-
-    // We don't free instvars itself here, because we have stuck
-    // it inside an SV *
-}
-
-
 // The rest of this file implements mapping Perl data structures
 // to and from MICO's Anys.
 
@@ -462,9 +291,6 @@ array_to_any (CORBA::Any *res, CORBA::TypeCode *tc, SV *sv)
 static bool
 except_to_any (CORBA::Any *res, CORBA::TypeCode *tc, SV *sv)
 {
-    SV **svp;
-    const char *id = tc->id();
-
     if (!res->except_put_begin(tc->id()))
 	return FALSE;
 
@@ -501,17 +327,13 @@ objref_to_any (CORBA::Any *res, CORBA::TypeCode *tc, SV *sv)
     if (!SvOK(sv))
 	return (*res <<= CORBA::Any::from_object (CORBA::Object::_nil(), tc->name()));
 
-    PMicoInstVars *iv = pmico_instvars_get (sv);
-
-    if (!iv && !sv_derived_from (sv, "CORBA::Object")) {
+    if (!sv_derived_from (sv, "CORBA::Object")) {
         warn ("Value is not a CORBA::Object");
-	return FALSE;
+	return false;
     }
 
-    if (!iv || !iv->obj)
-	iv = pmico_init_obj (sv, NULL, NULL, NULL, NULL, NULL);
-
-    return (*res <<= CORBA::Any::from_object (iv->obj, tc->name()));
+    return (*res <<= CORBA::Any::from_object ((CORBA::Object_ptr)SvIV((SV*)SvRV(sv)),
+					      tc->name()));
 }
 
 static CORBA::Long
@@ -523,7 +345,7 @@ union_find_arm (CORBA::TypeCode_ptr tc, SV *discriminator)
     CORBA::TypeCode_var dtype = tc->discriminator_type();
     CORBA::TCKind dkind = tc->discriminator_type()->kind();
 
-    CORBA::Long i = 0;
+    CORBA::ULong i = 0;
     bool found = false;
 
     for (i = 0; i<tc->member_count(); i++) {
@@ -649,7 +471,10 @@ static bool
 fixed_to_any (CORBA::Any *res, CORBA::TypeCode *tc, SV *sv)
 {
     int digits = tc->fixed_digits();
-    FixedBase::FixedValue val (digits+1);
+    int scale = tc->fixed_scale();
+    FixedBase fixed(digits, scale);
+    FixedBase::FixedValue val(digits + 1);
+
     int count;
     STRLEN len;
     char *str;
@@ -711,12 +536,18 @@ fixed_to_any (CORBA::Any *res, CORBA::TypeCode *tc, SV *sv)
       return FALSE;
     }
 
+    // FIXME: going through Fixed as opposed to FixedBase
+    //        (as we used to do, before MICO changed) loses,
+    //        since MICO just stores in integral types
+
     val.length (digits+1);
     val[digits] = (str[0] == '-');
     for (int i = 0 ; i < digits ; i++)
       val[i] = str[i+1] - '0';
+
+    fixed.from_digits (val);
     
-    return (*res <<= CORBA::Any::from_fixed (val, digits, tc->fixed_scale()));
+    return (*res <<= CORBA::Any::from_fixed (fixed, digits, scale));
 }
 
 static bool 
@@ -777,6 +608,7 @@ sv_to_any (CORBA::Any *res, CORBA::TypeCode *tc, SV *sv)
     case CORBA::tk_TypeCode:
     case CORBA::tk_Principal:
     case CORBA::tk_recursive:
+    default:
 	warn ("Unsupported output typecode %d\n", tc->kind());
 	return FALSE;
     }
@@ -1027,7 +859,7 @@ objref_from_any (CORBA::Any *any, CORBA::TypeCode *tc)
     CORBA::Object_ptr obj;
     *any >>= CORBA::Any::to_object (obj);
 
-    return pmico_find_or_create (obj);
+    return pmico_objref_to_sv (obj);
 }
 
 static SV *
@@ -1149,10 +981,16 @@ fixed_from_any (CORBA::Any *any, CORBA::TypeCode *tc)
     FixedBase::FixedValue_var v;
     CORBA::UShort digits = tc->fixed_digits();
     CORBA::Short scale = tc->fixed_scale();
+    FixedBase fixed (digits, scale);
 
     SV *sv = NULL;
 
-    if (*any >>= CORBA::Any::to_fixed (v, digits, scale)) {
+    // FIXME: going through Fixed as opposed to FixedBase
+    //        (as we used to do, before MICO changed) loses,
+    //         since MICO just stores in integral types
+    if (*any >>= CORBA::Any::to_fixed (fixed, digits, scale)) {
+
+	v = fixed.to_digits();
 
 	int i;
 	SV *tsv = newSV(digits+1);
